//|----------------------------------------------|
//| Mastered Espers Icon Hack                    |
//| by: madsiur                                  |
//| version: 1.1                                 |
//| Released on: March 27, 2022                  |
//| apply to: FF3us 1.0 (no header)              |
//|----------------------------------------------|

// assemble with bass v14, e.g. "bass -o rom.smc mastered.asm"
// https://www.romhacking.net/utilities/794/

//|----------------------------------------------|
//| variables                                    |
//|----------------------------------------------|

constant STAR_ENABLED(1)        // value of 1 to add star icon, other values disable insertion
constant ICON($7F)              // star icon value (can be changed to another icon)

//|----------------------------------------------|
//| functions                                    |
//|----------------------------------------------|

arch snes.cpu

// set base and origin
macro seek(variable offset) {
    origin (offset & $3FFFFF)
    base offset
}

//|----------------------------------------------|
//| star icon                                    |
//|----------------------------------------------|

if STAR_ENABLED == 1 {
    seek($C487B0)
    insert "star.bin"
}

//|----------------------------------------------|
//| bank $C3 code                                |
//|----------------------------------------------|

seek($C3031C)
jsl CHANGE_PALETTE		        // use yellow color for star icon,
                                // otherwise white color

// in init skills menu function

seek($C31B5E)
jsl SET_SPELL_OFFSET            // calculate actor's spell starting RAM offset

// in draw esper's name and MP cost function

seek($C3552E)
jsl CHECK_MASTERED              // check if current esper is mastered

seek($C35554)
jsl ADD_ICON                    // add star icon
nop
nop

seek($C3555D)
ldy #$000D                      // 13 spaces to blank

seek($C36BE8)
    jsl SET_PAL_34              // initialize yellow first
    nop
    nop

//|----------------------------------------------|
//| outside bank $C3 code                        |
//|----------------------------------------------|

seek($EEAF01)                   // you can change this free space offset
                                // as long as it's not in bank $C3
                                // and that you have 141 bytes of free space

// calculate actor's spell starting RAM offset

scope SET_SPELL_OFFSET: {
    tdc
    lda.b $28                   // load slot ID (0-3)
    tax                         // index it
    lda $69,x                   // loac actor ID in slot
    xba
    lda.b #$36                  // 54 spells
    rep #$20                    // 16-bit A
    sta.l $004202               // prepare multiplication (actor ID * 54)
    nop
    nop
    nop
    nop
    lda.l $004216               // load multiplication result
    sta.w $0203                 // save it
    sep #$20                    // 8-bit A
    tdc                         // clear A
    lda.b $28                   // load slot ID (0-3)
    tax                         // index it
    rtl
}

// check if current esper is mastered

scope CHECK_MASTERED: {
    phx
    phy
    tdc                         // clear A
    sta.b $FB                   // clear mastered esper byte
    lda $7E9D89,x               // load esper ID
    rep #$20                    // 16-bit A
    sta.b $FC                   // save it
    asl                         // x2
    sta.b $FE                   // save it
    asl                         // x4
    asl                         // x8
    clc
    adc.b $FE                   // x10
    clc
    adc.b $FC                   // x11
    tax
    stz.b $FC
    ldy.w #$0005                // 5 spells max per esper
    sep #$20                    // 8-bit A
loop:
    tdc                         // clear A
    lda $D86E01,x               // esper spell
    cmp #$FF                    // compare to null entry
    beq no_esper                // exit if no esper
    sta.b $FC                   // save spell ID
    rep #$20                    // 16-bit A
    lda.w $0203                 // load current character spell offset
    clc                         // clear carry
    adc.b $FC                   // spell offset + spell ID
    phx                         // save esper data index
    tax                         // set X as spell learnt percentage
    sep #$20                    // 8-bit A
    lda.w $1A6E,x               // load spell learnt percentage
    plx                         // restore esper data index
    cmp.b #$FF                  // compare learnt rate to 100%
    bne not_mastered            // branch if not 100%
    inx
    inx                         // go to next esper spell
    dey                         // decrement loop
    bne loop
no_esper:
    inc.b $FB                   // set esper as mastered
not_mastered:
    ply
    plx
    lda $7E9D89,x               // load esper ID
    rtl
}

// add star icon

scope ADD_ICON: {
    sta.w $2180                 // add ones digit to string
    lda.b $FB                   // load mastered esper byte
    beq not_mastered            // branch if not mastered
    lda.b #ICON                 // load icon ID
    bra end_string
not_mastered:
    lda.b #$FF                  // load space
end_string:
    sta.w $2180                 // add it to string
    stz.w $2180                 // end string
    rtl
}

// CHANGE_PALETTE function taken from Brave New World and adapted for this hack
// Credits: Gens, Ryo
// https://github.com/Gens81/brave-new-world/blob/master/BNW-post-2.1-modifications-main/ASM/new_status.asm#L1034-L1051
scope CHANGE_PALETTE: {
    cmp.b #ICON          		// star value?
    beq change            	    // branch if so
    lda.b $29               	// user palette
color_change:    
    sta [$EB],y           	    // save in Ram
    rtl
change: 
    lda #$34                    // yellow palette
    bra color_change            // go to save in ram
}

// SET_PAL_34 function taken from the Beyond Chaos randomizer
// Credits: Myria, C-Dude
scope SET_PAL_34: {
    ldx $00
    txa
    sta $2121
    rep #$20
    lda #$0000
    sta $7E30E9
    sta $7E30EB
    lda #$39CE
    sta $7E30ED
    lda #$03BF
    sta $7E30EF
    sep #$20
    rtl
}
